function player_init(player)
    player:set_name("Shiren")
    -- player:set_health(1000)
    player:set_element(Element.Summon)
    player:set_height(40.0)
    player:load_animation("battle.animation")
    player:set_texture(Resources.load_texture("battle.png"), false)
    local base_charge_color = Color.new(200, 200, 200, 255)
    player:set_fully_charged_color(base_charge_color)
    player:set_charge_position(0, -20)
    local arrow_regen_timer = 120
    local current_regen_timer = 0
    local special_regen_timer = 240
    local current_special_regen_timer = 0
    local arrows = 24
    local can_use_special = true

    player.on_update_func = function(self)
        if not can_use_special then
            if current_special_regen_timer < special_regen_timer then
                current_special_regen_timer =
                    current_special_regen_timer + 1
            else
                can_use_special = true
            end
        end

        -- Regain arrows up to a total of 24.
        if current_regen_timer >= arrow_regen_timer then
            arrows = arrows + 1
            if arrows > 24 then arrows = 24 end
            current_regen_timer = current_regen_timer - 40
        else
            current_regen_timer = current_regen_timer + 1
        end
    end

    player.normal_attack_func = function(player)
        current_regen_timer = 0
        local action = Action.new(player, "CHARACTER_SHOOT")
        action.on_execute_func = function(self, user)
            -- Spend arrows.
            if arrows > 1 then
                arrows = arrows - 1

                self:add_anim_action(1, function()
                    local props = HitProps.new(
                        player:attack_level() * 3,
                        Hit.Impact,
                        Element.None,
                        user:context(),
                        Drag.None
                    )
                    local t1 = player:get_tile(player:facing(), 1)
                    local arrow = create_arrow(player, props)
                    player:field():spawn(arrow, t1)
                end)
            end
        end
        return action
    end

    player.charged_attack_func = function(player)
        local action = Action.new(player, "CHARACTER_SWING")
        local tile = player:get_tile(player:facing(), 1)
        local up_tile = tile:get_tile(Direction.Up, 1)
        local down_tile = tile:get_tile(Direction.Down, 1)
        action.on_execute_func = function(self, user)
            local props = HitProps.new(
                (player:attack_level() * 20) - 10,
                Hit.Impact | Hit.Flinch | Hit.Flash,
                Element.Sword,
                user:context(),
                Drag.new(user:facing(), 1)
            )
            local slash1 = create_slash(user, props)
            local slash2 = create_slash(user, props)
            local slash3 = create_slash(user, props)
            self:add_anim_action(1, function()
                tile:set_highlight(Highlight.Flash)
                if up_tile and not up_tile:is_edge() then
                    up_tile:set_highlight(Highlight.Flash)
                end
                if down_tile and not down_tile:is_edge() then
                    down_tile:set_highlight(Highlight.Flash)
                end
                user:set_counterable(true)
            end)
            self:add_anim_action(3, function()
                user:field():spawn(slash1, tile)
                if up_tile and not up_tile:is_edge() then
                    user:field():spawn(slash2, up_tile)
                end
                if down_tile and not down_tile:is_edge() then
                    user:field():spawn(slash3, down_tile)
                end
            end)
            self:add_anim_action(5, function()
                tile:set_highlight(Highlight.None)
                up_tile:set_highlight(Highlight.None)
                down_tile:set_highlight(Highlight.None)
                if not slash1:deleted() then slash1:erase() end
                if not slash2:deleted() then slash2:erase() end
                if not slash3:deleted() then slash3:erase() end
                user:set_counterable(false)
            end)
        end
        return action
    end

    player.special_attack_func = function(player)
        if can_use_special then
            local action = Action.new(player, "CHARACTER_SWING")
            local tile = player:get_tile(player:facing(), 1)
            action.on_execute_func = function(self, user)
                local props = HitProps.new(
                    player:attack_level() * 5,
                    Hit.Impact | Hit.Flinch | Hit.Flash | Hit.Drag,
                    Element.Sword,
                    user:context(),
                    Drag.new(user:facing(), 1)
                )
                local slash = create_slash(user, props)
                self:add_anim_action(1, function()
                    tile:set_highlight(Highlight.Flash)
                    user:set_counterable(true)
                end)
                self:add_anim_action(3, function()
                    user:field():spawn(slash, tile)
                end)
                self:add_anim_action(5, function()
                    if not slash:deleted() then
                        slash:erase()
                    end
                    user:set_counterable(false)
                end)
                action.on_action_end_func = function(self)
                    can_use_special = false
                end
            end
            return action
        else
            return Action.new(player, "CHARACTER_IDLE")
        end
    end
end

function create_slash(user, props)
    local spell = Spell.new(user:team())
    spell:set_facing(user:facing())
    spell:set_hit_props(props)

    spell.on_update_func = function(self)
        self:current_tile():attack_entities(self)
    end

    spell.can_move_to_func = function(tile)
        return true
    end

    return spell
end

function create_arrow(user, props)
    local spell = Spell.new(user:team())
    spell:set_facing(user:facing())
    spell:set_offset(-30.0 * 0.5, -82.0 * 0.5)
    spell:set_texture(user:texture())
    spell:sprite():set_layer(-2)
    local anim = spell:animation()
    anim:copy_from(user:animation())
    anim:set_state("ARROW")
    anim:apply(spell:sprite())
    spell.slide_started = false
    local direction = spell:facing()
    spell:set_hit_props(props)
    spell:set_offset(0.0 * 0.5, -30.0 * 0.5)
    spell.on_update_func = function(self)
        self:current_tile():attack_entities(self)
        if self:is_sliding() == false then
            if self:current_tile():is_edge() and self.slide_started then
                self:delete()
            end

            local dest = self:get_tile(direction, 1)
            local ref = self
            self:slide(dest, 4,
                function()
                    ref.slide_started = true
                end
            )
        end
    end

    spell.can_move_to_func = function()
        return true
    end

    return spell
end
